/*
 * Avalon memory-mapped peripheral that generates VGA
 *
 * Stephen A. Edwards
 * Columbia University
 */

module vga_ball(input logic        clk,
	        input logic 	   reset,
		input logic [15:0]  writedata,
		input logic 	   write,
		input 		   chipselect,
		input logic [7:0]  address,

		output logic [7:0] VGA_R, VGA_G, VGA_B,
		output logic 	   VGA_CLK, VGA_HS, VGA_VS,
		                   VGA_BLANK_n,
		output logic 	   VGA_SYNC_n);

   logic [10:0]	   hcount;
   logic [9:0]     vcount;
   logic [24:0]    counter;		

   logic [15:0]    mushroom_out [0:15];
   logic [15:0]    mushroom_in [0:15];
   logic [15:0]    centipede_head_lef [0:15], centipede_head_dow [0:15];
   logic [0:15]    centipede_head_rig [0:15], centipede_eye_rig [0:15];
   logic [15:0]    centipede_eye_lef[0:15], centipede_eye_dow[0:15];
   logic [15:0]    centipede_feetone [0:15],centipede_feettwo [0:15],centipede_feetone_dow [0:15],centipede_feettwo_dow [0:15];
   logic [15:0]    bullet_figure [0:15];
   logic [15:0]    player_body [0:15];
   logic [15:0]    player_eye [0:15];
   logic [15:0]    spider_body [0:15];
   logic [15:0]    spider_eye [0:15];
   logic [15:0]    spider_legone [0:15];
   logic [15:0]    spider_legtwo [0:15];
   logic [0:15]    number_zero[0:15], number_one[0:15], number_two[0:15],number_three[0:15],number_four[0:15];
   logic [0:15]	   number_five[0:15],number_six[0:15],number_seven[0:15],number_eight[0:15],number_nine[0:15];

   logic [47:0]    mushroom_state [0:29];
   logic [15:0]    centipede_position_x[0:9], centipede_position_y[0:9], b_position_x[0:29], b_position_y[0:29];
   logic [15:0]    p_position_x, p_position_y, s_position_x, s_position_y;
   logic [3:0]	   title_state [0:39];
   logic [2:0]	   centipede_state[0:9];

   vga_counters counters(.clk50(clk), .*);
   moving_counters counterstwo(.clk50(clk), .*);

   always_ff @(posedge clk)
     if (reset) begin
	// initialization
  	// Todo
	//mushroom_state[0][0] <= 		1'b1;
  	mushroom_state[5][8] <= 		1'b1;
	mushroom_state[6][6] <= 		1'b1;
	mushroom_state[10][7] <= 		1'b1;
	mushroom_state[13][4] <= 		1'b1;
	mushroom_state[5][10] <= 		1'b1;
	mushroom_state[29][39] <= 		1'b1;
	mushroom_state[0][0] <= 		1'b1;
        centipede_position_x[0][15:0] <= 	16'd20;
        centipede_position_y[0][15:0] <= 	16'd20;
	centipede_state[0][2:0]<=		8'd 0;

	centipede_position_x[1][15:0] <= 	16'd40;
        centipede_position_y[1][15:0] <= 	16'd20;
	centipede_state[1][2:0]<=		8'd 1;

	centipede_position_x[2][15:0] <= 	16'd60;
        centipede_position_y[2][15:0] <= 	16'd20;
	centipede_state[2][2:0]<=		8'd 2;

	centipede_position_x[3][15:0] <= 	16'd80;
        centipede_position_y[3][15:0] <= 	16'd20;
	centipede_state[3][2:0]<=		8'd 3;

	centipede_position_x[4][15:0] <= 	16'd100;
        centipede_position_y[4][15:0] <= 	16'd20;
	centipede_state[4][2:0]<=		8'd 4;

	centipede_position_x[5][15:0] <= 	16'd120;
        centipede_position_y[5][15:0] <= 	16'd20;
	centipede_state[5][2:0]<=		8'd 5;

	centipede_position_x[6][15:0] <= 	16'd140;
        centipede_position_y[6][15:0] <= 	16'd20;
	centipede_state[6][2:0]<=		8'd 6;

        s_position_x[15:0] <= 			16'b0000000000011111;
	s_position_y[15:0] <= 			16'b0000000000011111;

	p_position_x[15:0] <= 			16'b0000000100011111;
	p_position_y[15:0] <= 			16'b0010000000011111;

        b_position_x[29][15:0] <= 	16'b0000000001111111;
        b_position_y[29][15:0] <= 	16'b0000000001111111;
	title_state[0][3:0]  <=		4'd 0;
	title_state[1][3:0]  <=		4'd 0;
	title_state[2][3:0]  <=		4'd 0;
	title_state[3][3:0]  <=		4'd 1;
	title_state[4][3:0]  <=		4'd 1;
	title_state[5][3:0]  <=		4'd 10;
	title_state[6][3:0]  <=		4'd 10;
	title_state[7][3:0]  <=		4'd 10;
	for(int i = 8; i < 14 ; i++)	title_state[i][3:0]  <=4'd 11;
	title_state[14][3:0]  <=	4'd 0;
	title_state[15][3:0]  <=	4'd 1;
	title_state[16][3:0]  <=	4'd 2;
	title_state[17][3:0]  <=	4'd 3;
	title_state[18][3:0]  <=	4'd 4;
	title_state[19][3:0]  <=	4'd 5;
	title_state[20][3:0]  <=	4'd 6;
	title_state[21][3:0]  <=	4'd 7;
	title_state[22][3:0]  <=	4'd 8;
	title_state[23][3:0]  <=	4'd 9;
	for( int j = 24; j < 40; j++)	title_state[j][3:0]  <=4'd 11;
	
	// mushroom outline object
	mushroom_out[0][15:0] <=	16'b0000111111110000;
	mushroom_out[1][15:0] <=	16'b0000111111110000;
	mushroom_out[2][15:0] <=	16'b0011000000001100;
	mushroom_out[3][15:0] <=	16'b0011000000001100;
	mushroom_out[4][15:0] <=	16'b1100000000000011;
	mushroom_out[5][15:0] <=	16'b1100000000000011;
	mushroom_out[6][15:0] <=	16'b1100000000000011;
	mushroom_out[7][15:0] <=	16'b1100000000000011;
	mushroom_out[8][15:0] <=	16'b1111111111111111;
	mushroom_out[9][15:0] <=	16'b1111111111111111;
	mushroom_out[10][15:0] <=	16'b0000110000110000;
	mushroom_out[11][15:0] <=	16'b0000110000110000;
	mushroom_out[12][15:0] <=	16'b0000110000110000;
	mushroom_out[13][15:0] <=	16'b0000110000110000;
	mushroom_out[14][15:0] <=	16'b0000111111110000;
	mushroom_out[15][15:0] <=	16'b0000111111110000;
	// mushroom inner object
	mushroom_in[0][15:0] <=		16'b0000000000000000;
	mushroom_in[1][15:0] <=		16'b0000000000000000;
	mushroom_in[2][15:0] <=		16'b0000111111110000;
	mushroom_in[3][15:0] <=		16'b0000111111110000;
	mushroom_in[4][15:0] <=		16'b0011111111111100;
	mushroom_in[5][15:0] <=		16'b0011111111111100;
	mushroom_in[6][15:0] <=		16'b0011111111111100;
	mushroom_in[7][15:0] <=		16'b0011111111111100;
	mushroom_in[8][15:0] <=		16'b0000000000000000;
	mushroom_in[9][15:0] <=		16'b0000000000000000;
	mushroom_in[10][15:0] <=	16'b0000001111000000;
	mushroom_in[11][15:0] <=	16'b0000001111000000;
	mushroom_in[12][15:0] <=	16'b0000001111000000;
	mushroom_in[13][15:0] <=	16'b0000001111000000;
	mushroom_in[14][15:0] <=	16'b0000000000000000;
	mushroom_in[15][15:0] <=	16'b0000000000000000;
	//centipede object
	centipede_head_lef[0][15:0] <=	16'b0000000000000000;
	centipede_head_lef[1][15:0] <=	16'b0000000000000000;
	centipede_head_lef[2][15:0] <=	16'b0000001111000000;
	centipede_head_lef[3][15:0] <=	16'b0000001111000000;
	centipede_head_lef[4][15:0] <=	16'b0000111111000011;
	centipede_head_lef[5][15:0] <=	16'b0000111111000011;
	centipede_head_lef[6][15:0] <=	16'b0011111111111111;
	centipede_head_lef[7][15:0] <=	16'b0011111111111111;
	centipede_head_lef[8][15:0] <=	16'b0011111111111111;
	centipede_head_lef[9][15:0] <=	16'b0011111111111111;
	centipede_head_lef[10][15:0] <=	16'b0000111111000011;
	centipede_head_lef[11][15:0] <=	16'b0000111111000011;
	centipede_head_lef[12][15:0] <=	16'b0000001111000000;
	centipede_head_lef[13][15:0] <=	16'b0000001111000000;
	centipede_head_lef[14][15:0] <=	16'b0000000000000000;
	centipede_head_lef[15][15:0] <=	16'b0000000000000000;
        //centipede_head_right
	centipede_head_rig[0][0:15] <=	16'b0000000000000000;
	centipede_head_rig[1][0:15] <=	16'b0000000000000000;
	centipede_head_rig[2][0:15] <=	16'b0000001111000000;
	centipede_head_rig[3][0:15] <=	16'b0000001111000000;
	centipede_head_rig[4][0:15] <=	16'b0000111111000011;
	centipede_head_rig[5][0:15] <=	16'b0000111111000011;
	centipede_head_rig[6][0:15] <=	16'b0011111111111111;
	centipede_head_rig[7][0:15] <=	16'b0011111111111111;
	centipede_head_rig[8][0:15] <=	16'b0011111111111111;
	centipede_head_rig[9][0:15] <=	16'b0011111111111111;
	centipede_head_rig[10][0:15] <=	16'b0000111111000011;
	centipede_head_rig[11][0:15] <=	16'b0000111111000011;
	centipede_head_rig[12][0:15] <=	16'b0000001111000000;
	centipede_head_rig[13][0:15] <=	16'b0000001111000000;
	centipede_head_rig[14][0:15] <=	16'b0000000000000000;
	centipede_head_rig[15][0:15] <=	16'b0000000000000000;

	centipede_head_dow[0][15:0] <=	16'b0000000000000000;
	centipede_head_dow[1][15:0] <=	16'b0000000000000000;
	centipede_head_dow[2][15:0] <=	16'b0000001111000000;
	centipede_head_dow[3][15:0] <=	16'b0000001111000000;
	centipede_head_dow[4][15:0] <=	16'b0000111111110000;
	centipede_head_dow[5][15:0] <=	16'b0000111111110000;
	centipede_head_dow[6][15:0] <=	16'b0011111111111100;
	centipede_head_dow[7][15:0] <=	16'b0011111111111100;
	centipede_head_dow[8][15:0] <=	16'b0011111111111100;
	centipede_head_dow[9][15:0] <=	16'b0011111111111100;
	centipede_head_dow[10][15:0] <=	16'b0000001111000000;
	centipede_head_dow[11][15:0] <=	16'b0000001111000000;
	centipede_head_dow[12][15:0] <=	16'b0000001111000000;
	centipede_head_dow[13][15:0] <=	16'b0000001111000000;
	centipede_head_dow[14][15:0] <=	16'b0000111111110000;
	centipede_head_dow[15][15:0] <=	16'b0000111111110000;

	//centipede eye
	centipede_eye_lef[0][15:0] <=	16'b0000000000000000;
	centipede_eye_lef[1][15:0] <=	16'b0000000000000000;
	centipede_eye_lef[2][15:0] <=	16'b0000000000111100;
	centipede_eye_lef[3][15:0] <=	16'b0000000000111100;
	centipede_eye_lef[4][15:0] <=	16'b0000000000111100;
	centipede_eye_lef[5][15:0] <=	16'b0000000000111100;
	centipede_eye_lef[6][15:0] <=	16'b0000000000000000;
	centipede_eye_lef[7][15:0] <=	16'b0000000000000000;
	centipede_eye_lef[8][15:0] <=	16'b0000000000000000;
	centipede_eye_lef[9][15:0] <=	16'b0000000000000000;
	centipede_eye_lef[10][15:0] <=	16'b0000000000111100;
	centipede_eye_lef[11][15:0] <=	16'b0000000000111100;
	centipede_eye_lef[12][15:0] <=	16'b0000000000111100;
	centipede_eye_lef[13][15:0] <=	16'b0000000000111100;
	centipede_eye_lef[14][15:0] <=	16'b0000000000000000;
	centipede_eye_lef[15][15:0] <=	16'b0000000000000000;

	centipede_eye_rig[0][0:15] <=	16'b0000000000000000;
	centipede_eye_rig[1][0:15] <=	16'b0000000000000000;
	centipede_eye_rig[2][0:15] <=	16'b0000000000111100;
	centipede_eye_rig[3][0:15] <=	16'b0000000000111100;
	centipede_eye_rig[4][0:15] <=	16'b0000000000111100;
	centipede_eye_rig[5][0:15] <=	16'b0000000000111100;
	centipede_eye_rig[6][0:15] <=	16'b0000000000000000;
	centipede_eye_rig[7][0:15] <=	16'b0000000000000000;
	centipede_eye_rig[8][0:15] <=	16'b0000000000000000;
	centipede_eye_rig[9][0:15] <=	16'b0000000000000000;
	centipede_eye_rig[10][0:15] <=	16'b0000000000111100;
	centipede_eye_rig[11][0:15] <=	16'b0000000000111100;
	centipede_eye_rig[12][0:15] <=	16'b0000000000111100;
	centipede_eye_rig[13][0:15] <=	16'b0000000000111100;
	centipede_eye_rig[14][0:15] <=	16'b0000000000000000;
	centipede_eye_rig[15][0:15] <=	16'b0000000000000000;

	centipede_eye_dow[0][15:0] <=	16'b0000000000000000;
	centipede_eye_dow[1][15:0] <=	16'b0000000000000000;
	centipede_eye_dow[2][15:0] <=	16'b0000000000000000;
	centipede_eye_dow[3][15:0] <=	16'b0000000000000000;
	centipede_eye_dow[4][15:0] <=	16'b0000000000000000;
	centipede_eye_dow[5][15:0] <=	16'b0000000000000000;
	centipede_eye_dow[6][15:0] <=	16'b0000000000000000;
	centipede_eye_dow[7][15:0] <=	16'b0000000000000000;
	centipede_eye_dow[8][15:0] <=	16'b0000000000000000;
	centipede_eye_dow[9][15:0] <=	16'b0000000000000000;
	centipede_eye_dow[10][15:0] <=	16'b0011110000111100;
	centipede_eye_dow[11][15:0] <=	16'b0011110000111100;
	centipede_eye_dow[12][15:0] <=	16'b0011110000111100;
	centipede_eye_dow[13][15:0] <=	16'b0011110000111100;
	centipede_eye_dow[14][15:0] <=	16'b0000000000000000;
	centipede_eye_dow[15][15:0] <=	16'b0000000000000000;

	//centipede feet1
	centipede_feetone[0][15:0] <=	16'b0000000001100000;
	centipede_feetone[1][15:0] <=	16'b0000000001100000;
	centipede_feetone[2][15:0] <=	16'b0000000000000000;
	centipede_feetone[3][15:0] <=	16'b0000000000000000;
	centipede_feetone[4][15:0] <=	16'b0000000000000000;
	centipede_feetone[5][15:0] <=	16'b0000000000000000;
	centipede_feetone[6][15:0] <=	16'b0000000000000000;
	centipede_feetone[7][15:0] <=	16'b0000000000000000;
	centipede_feetone[8][15:0] <=	16'b0000000000000000;
	centipede_feetone[9][15:0] <=	16'b0000000000000000;
	centipede_feetone[10][15:0] <=	16'b0000000000000000;
	centipede_feetone[11][15:0] <=	16'b0000000000000000;
	centipede_feetone[12][15:0] <=	16'b0000000000000000;
	centipede_feetone[13][15:0] <=	16'b0000000000000000;
	centipede_feetone[14][15:0] <=	16'b0000000001100000;
	centipede_feetone[15][15:0] <=	16'b0000000001100000;

	centipede_feetone_dow[0][15:0] <=	16'b0000000000000000;
	centipede_feetone_dow[1][15:0] <=	16'b0000000000000000;
	centipede_feetone_dow[2][15:0] <=	16'b0000000000000000;
	centipede_feetone_dow[3][15:0] <=	16'b0000000000000000;
	centipede_feetone_dow[4][15:0] <=	16'b0000000000000000;
	centipede_feetone_dow[5][15:0] <=	16'b0000000000000000;
	centipede_feetone_dow[6][15:0] <=	16'b0000000000000000;
	centipede_feetone_dow[7][15:0] <=	16'b0000000000000000;
	centipede_feetone_dow[8][15:0] <=	16'b0000000000000000;
	centipede_feetone_dow[9][15:0] <=	16'b1100000000000011;
	centipede_feetone_dow[10][15:0] <=	16'b1100000000000011;
	centipede_feetone_dow[11][15:0] <=	16'b0000000000000000;
	centipede_feetone_dow[12][15:0] <=	16'b0000000000000000;
	centipede_feetone_dow[13][15:0] <=	16'b0000000000000000;
	centipede_feetone_dow[14][15:0] <=	16'b0000000000000000;
	centipede_feetone_dow[15][15:0] <=	16'b0000000000000000;
	//centipede feet2
	centipede_feettwo[0][15:0] <=	16'b0000011000000000;
	centipede_feettwo[1][15:0] <=	16'b0000011000000000;
	centipede_feettwo[2][15:0] <=	16'b0000000000000000;
	centipede_feettwo[3][15:0] <=	16'b0000000000000000;
	centipede_feettwo[4][15:0] <=	16'b0000000000000000;
	centipede_feettwo[5][15:0] <=	16'b0000000000000000;
	centipede_feettwo[6][15:0] <=	16'b0000000000000000;
	centipede_feettwo[7][15:0] <=	16'b0000000000000000;
	centipede_feettwo[8][15:0] <=	16'b0000000000000000;
	centipede_feettwo[9][15:0] <=	16'b0000000000000000;
	centipede_feettwo[10][15:0] <=	16'b0000000000000000;
	centipede_feettwo[11][15:0] <=	16'b0000000000000000;
	centipede_feettwo[12][15:0] <=	16'b0000000000000000;
	centipede_feettwo[13][15:0] <=	16'b0000000000000000;
	centipede_feettwo[14][15:0] <=	16'b0000011000000000;
	centipede_feettwo[15][15:0] <=	16'b0000011000000000;

	centipede_feettwo_dow[0][15:0] <=	16'b0000000000000000;
	centipede_feettwo_dow[1][15:0] <=	16'b0000000000000000;
	centipede_feettwo_dow[2][15:0] <=	16'b0000000000000000;
	centipede_feettwo_dow[3][15:0] <=	16'b0000000000000000;
	centipede_feettwo_dow[4][15:0] <=	16'b0000000000000000;
	centipede_feettwo_dow[5][15:0] <=	16'b1100000000000011;
	centipede_feettwo_dow[6][15:0] <=	16'b1100000000000011;
	centipede_feettwo_dow[7][15:0] <=	16'b0000000000000000;
	centipede_feettwo_dow[8][15:0] <=	16'b0000000000000000;
	centipede_feettwo_dow[9][15:0] <=	16'b0000000000000000;
	centipede_feettwo_dow[10][15:0] <=	16'b0000000000000000;
	centipede_feettwo_dow[11][15:0] <=	16'b0000000000000000;
	centipede_feettwo_dow[12][15:0] <=	16'b0000000000000000;
	centipede_feettwo_dow[13][15:0] <=	16'b0000000000000000;
	centipede_feettwo_dow[14][15:0] <=	16'b0000000000000000;
	centipede_feettwo_dow[15][15:0] <=	16'b0000000000000000;
	//bullet object
	bullet_figure[0][15:0] <=	16'b0000000110000000;
	bullet_figure[1][15:0] <=	16'b0000000110000000;
	bullet_figure[2][15:0] <=	16'b0000000110000000;
	bullet_figure[3][15:0] <=	16'b0000000110000000;
	bullet_figure[4][15:0] <=	16'b0000000110000000;
	bullet_figure[5][15:0] <=	16'b0000000110000000;
	bullet_figure[6][15:0] <=	16'b0000000110000000;
	bullet_figure[7][15:0] <=	16'b0000000110000000;
	bullet_figure[8][15:0] <=	16'b0000000110000000;
	bullet_figure[9][15:0] <=	16'b0000000110000000;
	bullet_figure[10][15:0] <=	16'b0000000110000000;
	bullet_figure[11][15:0] <=	16'b0000000110000000;
	bullet_figure[12][15:0] <=	16'b0000000110000000;
	bullet_figure[13][15:0] <=	16'b0000000110000000;
	bullet_figure[14][15:0] <=	16'b0000000110000000;
	bullet_figure[15][15:0] <=	16'b0000000110000000;
	//player body
	player_body[0][15:0] <=		16'b0000000110000000;
	player_body[1][15:0] <=		16'b0000000110000000;
	player_body[2][15:0] <=		16'b0000011111100000;
	player_body[3][15:0] <=		16'b0000011111100000;
	player_body[4][15:0] <=		16'b0000000110000000;
	player_body[5][15:0] <=		16'b0000000110000000;
	player_body[6][15:0] <=		16'b0110000110000110;
	player_body[7][15:0] <=		16'b0110000110000110;
	player_body[8][15:0] <=		16'b0111111111111110;
	player_body[9][15:0] <=		16'b0111111111111110;
	player_body[10][15:0] <=	16'b0001111111111000;
	player_body[11][15:0] <=	16'b0001111111111000;
	player_body[12][15:0] <=	16'b0000011111100000;
	player_body[13][15:0] <=	16'b0000011111100000;
	player_body[14][15:0] <=	16'b0000011111100000;
	player_body[15][15:0] <=	16'b0000011111100000;
	//player eye
	player_eye[0][15:0] <=		16'b0000000000000000;
	player_eye[1][15:0] <=		16'b0000000000000000;
	player_eye[2][15:0] <=		16'b0000000000000000;
	player_eye[3][15:0] <=		16'b0000000000000000;
	player_eye[4][15:0] <=		16'b0001111001111000;
	player_eye[5][15:0] <=		16'b0001111001111000;
	player_eye[6][15:0] <=		16'b0001111001111000;
	player_eye[7][15:0] <=		16'b0001111001111000;
	player_eye[8][15:0] <=		16'b0000000000000000;
	player_eye[9][15:0] <=		16'b0000000000000000;
	player_eye[10][15:0] <=		16'b0000000000000000;
	player_eye[11][15:0] <=		16'b0000000000000000;
	player_eye[12][15:0] <=		16'b0000000000000000;
	player_eye[13][15:0] <=		16'b0000000000000000;
	player_eye[14][15:0] <=		16'b0000000000000000;
	player_eye[15][15:0] <=		16'b0000000000000000;
	//spider body
	spider_body[0][15:0] <=		16'b0000000000000000;
	spider_body[1][15:0] <=		16'b0000000000000000;
	spider_body[2][15:0] <=		16'b0000000000000000;
	spider_body[3][15:0] <=		16'b0000000000000000;
	spider_body[4][15:0] <=		16'b0000000000000000;
	spider_body[5][15:0] <=		16'b0000000000000000;
	spider_body[6][15:0] <=		16'b0000000000000000;
	spider_body[7][15:0] <=		16'b0000000000000000;
	spider_body[8][15:0] <=		16'b0000000000000000;
	spider_body[9][15:0] <=		16'b0000000000000000;
	spider_body[10][15:0] <=	16'b0000000100000000;
	spider_body[11][15:0] <=	16'b0000000100000000;
	spider_body[12][15:0] <=	16'b0000000100000000;
	spider_body[13][15:0] <=	16'b0000011111000000;
	spider_body[14][15:0] <=	16'b0000010001000000;
	spider_body[15][15:0] <=	16'b0000001010000000;
	//spider eye
	spider_eye[0][15:0] <=		16'b0000000000000000;
	spider_eye[1][15:0] <=		16'b0000000000000000;
	spider_eye[2][15:0] <=		16'b0000000000000000;
	spider_eye[3][15:0] <=		16'b0000000000000000;
	spider_eye[4][15:0] <=		16'b0000000000000000;
	spider_eye[5][15:0] <=		16'b0000000000000000;
	spider_eye[6][15:0] <=		16'b0000000000000000;
	spider_eye[7][15:0] <=		16'b0000000000000000;
	spider_eye[8][15:0] <=		16'b0000000000000000;
	spider_eye[9][15:0] <=		16'b0000000000000000;
	spider_eye[10][15:0] <=		16'b0000000000000000;
	spider_eye[11][15:0] <=		16'b0000001010000000;
	spider_eye[12][15:0] <=		16'b0000011011000000;
	spider_eye[13][15:0] <=		16'b0000000000000000;
	spider_eye[14][15:0] <=		16'b0000001110000000;
	spider_eye[15][15:0] <=		16'b0000000100000000;
	//spider legone
	spider_legone[0][15:0] <=	16'b0000000000000000;
	spider_legone[1][15:0] <=	16'b0000000000000000;
	spider_legone[2][15:0] <=	16'b0000000000000000;
	spider_legone[3][15:0] <=	16'b0000000000000000;
	spider_legone[4][15:0] <=	16'b0000000000000000;
	spider_legone[5][15:0] <=	16'b0000000000000000;
	spider_legone[6][15:0] <=	16'b0000000000000000;
	spider_legone[7][15:0] <=	16'b0000000000000000;
	spider_legone[8][15:0] <=	16'b0000000000000000;
	spider_legone[9][15:0] <=	16'b0000000000000000;
	spider_legone[10][15:0] <=	16'b0011100000111000;
	spider_legone[11][15:0] <=	16'b0100010001000100;
	spider_legone[12][15:0] <=	16'b1000000000000010;
	spider_legone[13][15:0] <=	16'b0011000000011000;
	spider_legone[14][15:0] <=	16'b0100100000100100;
	spider_legone[15][15:0] <=	16'b1000000000000010;
	//spider legtwo
	spider_legtwo[0][15:0] <=	16'b0000000000000000;
	spider_legtwo[1][15:0] <=	16'b0000000000000000;
	spider_legtwo[2][15:0] <=	16'b0000000000000000;
	spider_legtwo[3][15:0] <=	16'b0000000000000000;
	spider_legtwo[4][15:0] <=	16'b0000000000000000;
	spider_legtwo[5][15:0] <=	16'b0000000000000000;
	spider_legtwo[6][15:0] <=	16'b0000000000000000;
	spider_legtwo[7][15:0] <=	16'b0011000000011000;
	spider_legtwo[8][15:0] <=	16'b0100100000100100;
	spider_legtwo[9][15:0] <=	16'b0100100000100100;
	spider_legtwo[10][15:0] <=	16'b1000010001000010;
	spider_legtwo[11][15:0] <=	16'b1011010001011010;
	spider_legtwo[12][15:0] <=	16'b0011000000011000;
	spider_legtwo[13][15:0] <=	16'b0100100000100100;
	spider_legtwo[14][15:0] <=	16'b1000100000100010;
	spider_legtwo[15][15:0] <=	16'b0000000000000000;
	//number0
	number_zero[0][0:15] <=		16'b0000000000000000;
	number_zero[1][0:15] <=		16'b0000000000000000;
	number_zero[2][0:15] <=		16'b0000000000000000;
	number_zero[3][0:15] <=		16'b0000011111100000;
	number_zero[4][0:15] <=		16'b0000111111110000;
	number_zero[5][0:15] <=		16'b0000110000110000;
	number_zero[6][0:15] <=		16'b0000110000110000;
	number_zero[7][0:15] <=		16'b0000110000110000;
	number_zero[8][0:15] <=		16'b0000110000110000;
	number_zero[9][0:15] <=		16'b0000110000110000;
	number_zero[10][0:15] <=	16'b0000110000110000;
	number_zero[11][0:15] <=	16'b0000111111110000;
	number_zero[12][0:15] <=	16'b0000011111100000;
	number_zero[13][0:15] <=	16'b0000000000000000;
	number_zero[14][0:15] <=	16'b0000000000000000;
	number_zero[15][0:15] <=	16'b0000000000000000;
	//number1
	number_one[0][0:15] <=		16'b0000000000000000;
	number_one[1][0:15] <=		16'b0000000000000000;
	number_one[2][0:15] <=		16'b0000000000000000;
	number_one[3][0:15] <=		16'b0000000110000000;
	number_one[4][0:15] <=		16'b0000001110000000;
	number_one[5][0:15] <=		16'b0000011110000000;
	number_one[6][0:15] <=		16'b0000000110000000;
	number_one[7][0:15] <=		16'b0000000110000000;
	number_one[8][0:15] <=		16'b0000000110000000;
	number_one[9][0:15] <=		16'b0000000110000000;
	number_one[10][0:15] <=		16'b0000000110000000;
	number_one[11][0:15] <=		16'b0000011111100000;
	number_one[12][0:15] <=		16'b0000011111100000;
	number_one[13][0:15] <=		16'b0000000000000000;
	number_one[14][0:15] <=		16'b0000000000000000;
	number_one[15][0:15] <=		16'b0000000000000000;
	//number2
	number_two[0][0:15] <=		16'b0000000000000000;
	number_two[1][0:15] <=		16'b0000000000000000;
	number_two[2][0:15] <=		16'b0000000000000000;
	number_two[3][0:15] <=		16'b0000011111100000;
	number_two[4][0:15] <=		16'b0000111111110000;
	number_two[5][0:15] <=		16'b0000110000110000;
	number_two[6][0:15] <=		16'b0000000001110000;
	number_two[7][0:15] <=		16'b0000000011100000;
	number_two[8][0:15] <=		16'b0000000111000000;
	number_two[9][0:15] <=		16'b0000001110000000;
	number_two[10][0:15] <=		16'b0000011100000000;
	number_two[11][0:15] <=		16'b0000111111110000;
	number_two[12][0:15] <=		16'b0000111111110000;
	number_two[13][0:15] <=		16'b0000000000000000;
	number_two[14][0:15] <=		16'b0000000000000000;
	number_two[15][0:15] <=		16'b0000000000000000;
	//number3
	number_three[0][0:15] <=	16'b0000000000000000;
	number_three[1][0:15] <=	16'b0000000000000000;
	number_three[2][0:15] <=	16'b0000000000000000;
	number_three[3][0:15] <=	16'b0000011111100000;
	number_three[4][0:15] <=	16'b0000111111110000;
	number_three[5][0:15] <=	16'b0000110000110000;
	number_three[6][0:15] <=	16'b0000000000110000;
	number_three[7][0:15] <=	16'b0000000111100000;
	number_three[8][0:15] <=	16'b0000000111100000;
	number_three[9][0:15] <=	16'b0000000000110000;
	number_three[10][0:15] <=	16'b0000110000110000;
	number_three[11][0:15] <=	16'b0000111111110000;
	number_three[12][0:15] <=	16'b0000011111100000;
	number_three[13][0:15] <=	16'b0000000000000000;
	number_three[14][0:15] <=	16'b0000000000000000;
	number_three[15][0:15] <=	16'b0000000000000000;
	//number4
	number_four[0][0:15] <=		16'b0000000000000000;
	number_four[1][0:15] <=		16'b0000000000000000;
	number_four[2][0:15] <=		16'b0000000000000000;
	number_four[3][0:15] <=		16'b0000000011100000;
	number_four[4][0:15] <=		16'b0000000111100000;
	number_four[5][0:15] <=		16'b0000001111100000;
	number_four[6][0:15] <=		16'b0000011101100000;
	number_four[7][0:15] <=		16'b0000111001100000;
	number_four[8][0:15] <=		16'b0000110001100000;
	number_four[9][0:15] <=		16'b0000111111110000;
	number_four[10][0:15] <=	16'b0000111111110000;
	number_four[11][0:15] <=	16'b0000000001100000;
	number_four[12][0:15] <=	16'b0000000001100000;
	number_four[13][0:15] <=	16'b0000000000000000;
	number_four[14][0:15] <=	16'b0000000000000000;
	number_four[15][0:15] <=	16'b0000000000000000;
	//number5
	number_five[0][0:15] <=		16'b0000000000000000;
	number_five[1][0:15] <=		16'b0000000000000000;
	number_five[2][0:15] <=		16'b0000000000000000;
	number_five[3][0:15] <=		16'b0000111111110000;
	number_five[4][0:15] <=		16'b0000111111110000;
	number_five[5][0:15] <=		16'b0000110000000000;
	number_five[6][0:15] <=		16'b0000110000000000;
	number_five[7][0:15] <=		16'b0000111111100000;
	number_five[8][0:15] <=		16'b0000111111110000;
	number_five[9][0:15] <=		16'b0000000000110000;
	number_five[10][0:15] <=	16'b0000000000110000;
	number_five[11][0:15] <=	16'b0000111111110000;
	number_five[12][0:15] <=	16'b0000111111100000;
	number_five[13][0:15] <=	16'b0000000000000000;
	number_five[14][0:15] <=	16'b0000000000000000;
	number_five[15][0:15] <=	16'b0000000000000000;
	//number6
	number_six[0][0:15] <=		16'b0000000000000000;
	number_six[1][0:15] <=		16'b0000000000000000;
	number_six[2][0:15] <=		16'b0000000000000000;
	number_six[3][0:15] <=		16'b0000011111110000;
	number_six[4][0:15] <=		16'b0000111111110000;
	number_six[5][0:15] <=		16'b0000110000000000;
	number_six[6][0:15] <=		16'b0000110000000000;
	number_six[7][0:15] <=		16'b0000111111100000;
	number_six[8][0:15] <=		16'b0000111111110000;
	number_six[9][0:15] <=		16'b0000110000110000;
	number_six[10][0:15] <=		16'b0000110000110000;
	number_six[11][0:15] <=		16'b0000111111110000;
	number_six[12][0:15] <=		16'b0000011111100000;
	number_six[13][0:15] <=		16'b0000000000000000;
	number_six[14][0:15] <=		16'b0000000000000000;
	number_six[15][0:15] <=		16'b0000000000000000;
	//number7
	number_seven[0][0:15] <=	16'b0000000000000000;
	number_seven[1][0:15] <=	16'b0000000000000000;
	number_seven[2][0:15] <=	16'b0000000000000000;
	number_seven[3][0:15] <=	16'b0000111111110000;
	number_seven[4][0:15] <=	16'b0000111111110000;
	number_seven[5][0:15] <=	16'b0000000000110000;
	number_seven[6][0:15] <=	16'b0000000001110000;
	number_seven[7][0:15] <=	16'b0000000011100000;
	number_seven[8][0:15] <=	16'b0000000111000000;
	number_seven[9][0:15] <=	16'b0000001110000000;
	number_seven[10][0:15] <=	16'b0000011100000000;
	number_seven[11][0:15] <=	16'b0000111000000000;
	number_seven[12][0:15] <=	16'b0000110000000000;
	number_seven[13][0:15] <=	16'b0000000000000000;
	number_seven[14][0:15] <=	16'b0000000000000000;
	number_seven[15][0:15] <=	16'b0000000000000000;
	//number8
	number_eight[0][0:15] <=	16'b0000000000000000;
	number_eight[1][0:15] <=	16'b0000000000000000;
	number_eight[2][0:15] <=	16'b0000000000000000;
	number_eight[3][0:15] <=	16'b0000011111100000;
	number_eight[4][0:15] <=	16'b0000111111110000;
	number_eight[5][0:15] <=	16'b0000110000110000;
	number_eight[6][0:15] <=	16'b0000110000110000;
	number_eight[7][0:15] <=	16'b0000011111100000;
	number_eight[8][0:15] <=	16'b0000011111100000;
	number_eight[9][0:15] <=	16'b0000110000110000;
	number_eight[10][0:15] <=	16'b0000110000110000;
	number_eight[11][0:15] <=	16'b0000111111110000;
	number_eight[12][0:15] <=	16'b0000011111100000;
	number_eight[13][0:15] <=	16'b0000000000000000;
	number_eight[14][0:15] <=	16'b0000000000000000;
	number_eight[15][0:15] <=	16'b0000000000000000;
	//number9
	number_nine[0][0:15] <=		16'b0000000000000000;
	number_nine[1][0:15] <=		16'b0000000000000000;
	number_nine[2][0:15] <=		16'b0000000000000000;
	number_nine[3][0:15] <=		16'b0000011111100000;
	number_nine[4][0:15] <=		16'b0000111111110000;
	number_nine[5][0:15] <=		16'b0000110000110000;
	number_nine[6][0:15] <=		16'b0000110000110000;
	number_nine[7][0:15] <=		16'b0000111111110000;
	number_nine[8][0:15] <=		16'b0000011111110000;
	number_nine[9][0:15] <=		16'b0000000000110000;
	number_nine[10][0:15] <=	16'b0000000000110000;
	number_nine[11][0:15] <=	16'b0000111111110000;
	number_nine[12][0:15] <=	16'b0000111111100000;
	number_nine[13][0:15] <=	16'b0000000000000000;
	number_nine[14][0:15] <=	16'b0000000000000000;
	number_nine[15][0:15] <=	16'b0000000000000000;



	
	//mushroom 3 * 30 
        //centipede 10 * 2 
        //player 1 * 2
        //spider 1 * 2
        //bullet 30 * 2
     end else if (chipselect && write) begin
		if(address <90) begin
			case(address % 3)
			8'd0: mushroom_state[(address / 3)][15: 0] <= writedata;
			8'd1: mushroom_state[(address / 3)][31: 16] <= writedata;
			8'd2: mushroom_state[(address / 3)][47: 32] <= writedata;
			endcase
		end
		else if(address >= 90 && address < 110) begin
			if(address[0] == 0)
				centipede_position_x[(address - 90)/2][15:0] <= writedata;
			else
				centipede_position_y[(address - 91)/2][15:0] <= writedata;
		end
		else if(address >= 110 && address < 170) begin
			if(address[0] == 0)
				b_position_x[(address - 110)/2][15:0] <= writedata;
			else			
				b_position_y[(address - 111)/2][15:0] <= writedata;
		end
		else if (address >180 && address <= 220) begin
				title_state[address - 181][3:0] <= writedata;
		end
		else if (address >= 221 && address <= 230) begin
				centipede_state[address - 221][2:0] <= writedata;
		end
		else begin
                	case(address) 
			8'd170: p_position_x <= writedata;
			8'd171: p_position_y <= writedata;
			8'd172: s_position_x <= writedata;
			8'd173: s_position_y <= writedata;
			endcase
		end
	end
    

   always_comb begin
      {VGA_R, VGA_G, VGA_B} = {8'h0, 8'h0, 8'h0};
      if (VGA_BLANK_n ) begin
	// Draw Mushroom	
			if(mushroom_state[vcount[9:0]>>4][hcount[10:1]>>4] == 1) begin
				
			        //if(hcount[10:1] - 128 < 16 && vcount[9:0] - (i << 4) < 16) begin				
					if(mushroom_out[vcount[3:0]][hcount[4:1]] == 1)
						{VGA_R, VGA_G, VGA_B} = {8'hff, 8'h00, 8'h00}; //red
					else if(mushroom_in[vcount[3:0]][hcount[4:1]] == 1)
						{VGA_R, VGA_G, VGA_B} = {8'hff, 8'hcc, 8'h00}; //orange
					else
						{VGA_R, VGA_G, VGA_B} = {8'h0, 8'h0, 8'h0};
				//end
			end
			
	//Draw title
			else if(title_state[hcount[10:1]>>4][3:0] == 0 && number_zero[vcount[3:0]][hcount[4:1]] == 1 && vcount < 16)
				{VGA_R, VGA_G, VGA_B} = {8'hff, 8'h00, 8'h00};
			else if(title_state[hcount[10:1]>>4][3:0] == 1 && number_one[vcount[3:0]][hcount[4:1]] == 1 && vcount < 16)
				{VGA_R, VGA_G, VGA_B} = {8'hff, 8'h00, 8'h00};
			else if(title_state[hcount[10:1]>>4][3:0] == 2 && number_two[vcount[3:0]][hcount[4:1]] == 1 && vcount < 16)
				{VGA_R, VGA_G, VGA_B} = {8'hff, 8'h00, 8'h00};	
			else if(title_state[hcount[10:1]>>4][3:0] == 3 && number_three[vcount[3:0]][hcount[4:1]] == 1 && vcount < 16)
				{VGA_R, VGA_G, VGA_B} = {8'hff, 8'h00, 8'h00};	
			else if(title_state[hcount[10:1]>>4][3:0] == 4 && number_four[vcount[3:0]][hcount[4:1]] == 1 && vcount < 16)
				{VGA_R, VGA_G, VGA_B} = {8'hff, 8'h00, 8'h00};	
			else if(title_state[hcount[10:1]>>4][3:0] == 5 && number_five[vcount[3:0]][hcount[4:1]] == 1 && vcount < 16)
				{VGA_R, VGA_G, VGA_B} = {8'hff, 8'h00, 8'h00};	
			else if(title_state[hcount[10:1]>>4][3:0] == 6 && number_six[vcount[3:0]][hcount[4:1]] == 1 && vcount < 16)
				{VGA_R, VGA_G, VGA_B} = {8'hff, 8'h00, 8'h00};	
			else if(title_state[hcount[10:1]>>4][3:0] == 7 && number_seven[vcount[3:0]][hcount[4:1]] == 1 && vcount < 16)
				{VGA_R, VGA_G, VGA_B} = {8'hff, 8'h00, 8'h00};	
			else if(title_state[hcount[10:1]>>4][3:0] == 8 && number_eight[vcount[3:0]][hcount[4:1]] == 1 && vcount < 16)
				{VGA_R, VGA_G, VGA_B} = {8'hff, 8'h00, 8'h00};	
			else if(title_state[hcount[10:1]>>4][3:0] == 9 && number_nine[vcount[3:0]][hcount[4:1]] == 1 && vcount < 16)
				{VGA_R, VGA_G, VGA_B} = {8'hff, 8'h00, 8'h00};	
			else if(title_state[hcount[10:1]>>4][3:0] == 10  && vcount < 16)
				if(player_body[vcount[3:0]][hcount[4:1]] == 1)
					{VGA_R, VGA_G, VGA_B} = {8'hff, 8'hff, 8'hff};
				else if(player_eye[vcount[3:0]][hcount[4:1]] == 1)
					{VGA_R, VGA_G, VGA_B} = {8'hff, 8'h00, 8'h00};
			else begin
				{VGA_R, VGA_G, VGA_B} = {8'h00, 8'h00, 8'h00};	
			end
	

	// Draw Centipede
	for(int m = 0; m < 10; m++) begin
        	if(hcount[10:1] - centipede_position_x[m] < 16 && vcount[9:0] - centipede_position_y[m] < 16) begin
     			if(centipede_head_lef[vcount[9:0] - centipede_position_y[m]][hcount[10:1] - centipede_position_x[m]] == 1 && (centipede_state[m][2:0] == 0 || centipede_state[m][2:0]==1))  //left
					{VGA_R, VGA_G, VGA_B} = {8'h00, 8'haa, 8'h00};   //green
			else if(centipede_eye_lef[vcount[9:0] - centipede_position_y[m]][hcount[10:1] - centipede_position_x[m]] == 1 && centipede_state[m][2:0] == 0)  //body 
					{VGA_R, VGA_G, VGA_B} = {8'h00, 8'haa, 8'h00};   //green
			else if(centipede_eye_lef[vcount[9:0] - centipede_position_y[m]][hcount[10:1] - centipede_position_x[m]] == 1 && centipede_state[m][2:0] == 1) //eye 
					{VGA_R, VGA_G, VGA_B} = {8'hff, 8'h00, 8'h00};   // red

			else if(centipede_head_rig[vcount[9:0] - centipede_position_y[m]][hcount[10:1] - centipede_position_x[m]] == 1 && (centipede_state[m][2:0] == 2 || centipede_state[m][2:0]==3))  //right
					{VGA_R, VGA_G, VGA_B} = {8'h00, 8'haa, 8'h00};   //green
			else if(centipede_eye_rig[vcount[9:0] - centipede_position_y[m]][hcount[10:1] - centipede_position_x[m]] == 1 && centipede_state[m][2:0] == 2)  //body 
					{VGA_R, VGA_G, VGA_B} = {8'h00, 8'haa, 8'h00};   //green
			else if(centipede_eye_rig[vcount[9:0] - centipede_position_y[m]][hcount[10:1] - centipede_position_x[m]] == 1 && centipede_state[m][2:0] == 3) //eye 
					{VGA_R, VGA_G, VGA_B} = {8'hff, 8'h00, 8'h00};   // red
						
			else if(centipede_head_dow[vcount[9:0] - centipede_position_y[m]][hcount[10:1] - centipede_position_x[m]] == 1 && (centipede_state[m][2:0] == 4 || centipede_state[m][2:0]==5))  //right
					{VGA_R, VGA_G, VGA_B} = {8'h00, 8'haa, 8'h00};   //green
			else if(centipede_eye_dow[vcount[9:0] - centipede_position_y[m]][hcount[10:1] - centipede_position_x[m]] == 1 && centipede_state[m][2:0] == 4)  //body 
					{VGA_R, VGA_G, VGA_B} = {8'h00, 8'haa, 8'h00};   //green
			else if(centipede_eye_dow[vcount[9:0] - centipede_position_y[m]][hcount[10:1] - centipede_position_x[m]] == 1 && centipede_state[m][2:0] == 5) //eye 
					{VGA_R, VGA_G, VGA_B} = {8'hff, 8'h00, 8'h00};   // red			
			else if(centipede_feetone[vcount[9:0] - centipede_position_y[m]][hcount[10:1] - centipede_position_x[m]] == 1 && counter[20]==0 && (centipede_state[m][2:0] == 0 || centipede_state[m][2:0]==1 || centipede_state[m][2:0] == 2 || centipede_state[m][2:0]==3))
					{VGA_R, VGA_G, VGA_B} = {8'hff, 8'hff, 8'hff};
			else if(centipede_feettwo[vcount[9:0] - centipede_position_y[m]][hcount[10:1] - centipede_position_x[m]] == 1 && counter[20]==1 && (centipede_state[m][2:0] == 0 || centipede_state[m][2:0]==1 || centipede_state[m][2:0] == 2 || centipede_state[m][2:0]==3))
					{VGA_R, VGA_G, VGA_B} = {8'hff, 8'hff, 8'hff};
			else if(centipede_feetone_dow[vcount[9:0] - centipede_position_y[m]][hcount[10:1] - centipede_position_x[m]] == 1 && counter[20]==0 && (centipede_state[m][2:0] == 4 || centipede_state[m][2:0]==5))
					{VGA_R, VGA_G, VGA_B} = {8'hff, 8'hff, 8'hff};
			else if(centipede_feettwo_dow[vcount[9:0] - centipede_position_y[m]][hcount[10:1] - centipede_position_x[m]] == 1 && counter[20]==1 && (centipede_state[m][2:0] == 4 || centipede_state[m][2:0]== 5))
					{VGA_R, VGA_G, VGA_B} = {8'hff, 8'hff, 8'hff};
			else {VGA_R, VGA_G, VGA_B} = {8'h0, 8'h0, 8'h0};
		end
	end
        // Draw Bullet
	for(int n = 0; n < 30; n++) begin
        	if(hcount[10:1] - b_position_x[n] < 16 && vcount[9:0] - b_position_y[n] < 16) begin
     			if(bullet_figure[vcount[9:0] - b_position_y[n]][hcount[10:1] - b_position_x[n]] == 1)
					{VGA_R, VGA_G, VGA_B} = {8'hff, 8'h00, 8'h00}; //red
		end
	end
	// Draw player
	if(hcount[10:1] - p_position_x < 16 && vcount[9:0] - p_position_y < 16) begin
     			if(player_body[vcount[9:0] - p_position_y][hcount[10:1] - p_position_x] == 1)
					{VGA_R, VGA_G, VGA_B} = {8'hff, 8'hff, 8'hff};  //white
			else if(player_eye[vcount[9:0] - p_position_y][hcount[10:1] - p_position_x] == 1)
					{VGA_R, VGA_G, VGA_B} = {8'hff, 8'h00, 8'h00};  //red
	end
	// Draw Spider
	if(hcount[10:1] - s_position_x < 16 && vcount[9:0] - s_position_y < 16) begin
			if(spider_body[vcount[9:0] - s_position_y][hcount[10:1] - s_position_x] == 1)
					{VGA_R, VGA_G, VGA_B} = {8'h00, 8'haa, 8'h00};  //green
			else if(spider_eye[vcount[9:0] - s_position_y][hcount[10:1] - s_position_x] == 1)
					{VGA_R, VGA_G, VGA_B} = {8'hff, 8'h00, 8'h00};  //red
     			else if(spider_legone[vcount[9:0] - s_position_y][hcount[10:1] - s_position_x] == 1 && counter[24]==0)
					{VGA_R, VGA_G, VGA_B} = {8'hff, 8'hff, 8'hff}; 
			else if(spider_legtwo[vcount[9:0] - s_position_y][hcount[10:1] - s_position_x] == 1 && counter[24]==1)
					{VGA_R, VGA_G, VGA_B} = {8'hff, 8'hff, 8'hff}; 
	end      
      end
   end
	       
endmodule

module moving_counters(
		input logic 	     clk50, reset,
 		output logic [24:0]  counter
			);
always_ff @(posedge clk50 or posedge reset)
     if (reset)          counter <= 0;
     else  	         counter <= counter + 25'd 1;


endmodule




module vga_counters(
 input logic 	     clk50, reset,
 output logic [10:0] hcount,  // hcount[10:1] is pixel column
 output logic [9:0]  vcount,  // vcount[9:0] is pixel row
 output logic 	     VGA_CLK, VGA_HS, VGA_VS, VGA_BLANK_n, VGA_SYNC_n);

/*
 * 640 X 480 VGA timing for a 50 MHz clock: one pixel every other cycle
 * 
 * HCOUNT 1599 0             1279       1599 0
 *             _______________              ________
 * ___________|    Video      |____________|  Video
 * 
 * 
 * |SYNC| BP |<-- HACTIVE -->|FP|SYNC| BP |<-- HACTIVE
 *       _______________________      _____________
 * |____|       VGA_HS          |____|
 */
   // Parameters for hcount
   parameter HACTIVE      = 11'd 1280,
             HFRONT_PORCH = 11'd 32,
             HSYNC        = 11'd 192,
             HBACK_PORCH  = 11'd 96,   
             HTOTAL       = HACTIVE + HFRONT_PORCH + HSYNC +
                            HBACK_PORCH; // 1600
   
   // Parameters for vcount
   parameter VACTIVE      = 10'd 480,
             VFRONT_PORCH = 10'd 10,
             VSYNC        = 10'd 2,
             VBACK_PORCH  = 10'd 33,
             VTOTAL       = VACTIVE + VFRONT_PORCH + VSYNC +
                            VBACK_PORCH; // 525

   logic endOfLine;
   
   always_ff @(posedge clk50 or posedge reset)
     if (reset)          hcount <= 0;
     else if (endOfLine) hcount <= 0;
     else  	         hcount <= hcount + 11'd 1;

   assign endOfLine = hcount == HTOTAL - 1;
       
   logic endOfField;
   
   always_ff @(posedge clk50 or posedge reset)
     if (reset)          vcount <= 0;
     else if (endOfLine)
       if (endOfField)   vcount <= 0;
       else              vcount <= vcount + 10'd 1;

   assign endOfField = vcount == VTOTAL - 1;

   // Horizontal sync: from 0x520 to 0x5DF (0x57F)
   // 101 0010 0000 to 101 1101 1111
   assign VGA_HS = !( (hcount[10:8] == 3'b101) &
		      !(hcount[7:5] == 3'b111));
   assign VGA_VS = !( vcount[9:1] == (VACTIVE + VFRONT_PORCH) / 2);

   assign VGA_SYNC_n = 1'b0; // For putting sync on the green signal; unused
   
   // Horizontal active: 0 to 1279     Vertical active: 0 to 479
   // 101 0000 0000  1280	       01 1110 0000  480
   // 110 0011 1111  1599	       10 0000 1100  524
   assign VGA_BLANK_n = !( hcount[10] & (hcount[9] | hcount[8]) ) &
			!( vcount[9] | (vcount[8:5] == 4'b1111) );

   /* VGA_CLK is 25 MHz
    *             __    __    __
    * clk50    __|  |__|  |__|
    *        
    *             _____       __
    * hcount[0]__|     |_____|
    */
   assign VGA_CLK = hcount[0]; // 25 MHz clock: rising edge sensitive
   
endmodule
