

#include <linux/module.h>
#include <linux/init.h>
#include <linux/errno.h>
#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/platform_device.h>
#include <linux/miscdevice.h>
#include <linux/slab.h>
#include <linux/io.h>
#include <linux/of.h>
#include <linux/of_address.h>
#include <linux/fs.h>
#include <linux/uaccess.h>
#include "vga_ball.h"

#define DRIVER_NAME "vga_ball"

/* Device registers */
#define BG(x) (x)  //mushroom
#define C(x) ((x)+2*90) //centipede
#define B(x) ((x)+2*90+2*20) //bullet
#define P(x) ((x)+2*90+2*20+2*60) //player
#define S(x) ((x)+2*90+2*20+2*60+4) //spider
#define title(x) ((x)+ 181*2) //title
#define state(x) ((x)+ 221*2) //centipede's state

//mouse registers
#define xlatch(x) ((x)+176*2)
#define ylatch(x) ((x)+177*2)
#define leflatch(x) ((x)+178*2)
#define riglatch(x) ((x)+179*2)
#define midlatch(x) ((x)+180*2)
#define start(x) ((x)+174*2)

/*
 * Information about our device
 */
struct vga_ball_dev {
	struct resource res; /* Resource: our registers */
	void __iomem *virtbase; /* Where registers can be accessed in memory */
    vga_ball_background mushroom;
    vga_ball_title title;
    vga_ball_centipede centipede;
    vga_ball_state state;
    vga_ball_bullet bullet;
    vga_ball_object player;
    vga_ball_object spider;
    vga_ball_start start;
    vga_ball_mouse mouse;
} dev;

/*
 * Write segments of a single digit
 * Assumes digit is in range and the device information has been set up
 */
/*static void write_background(vga_ball_color_t *background)
{
	iowrite8(background->red, BG_RED(dev.virtbase) );
	iowrite8(background->green, BG_GREEN(dev.virtbase) );
	iowrite8(background->blue, BG_BLUE(dev.virtbase) );
	dev.background = *background;
}
*/
static void write_background(vga_ball_background *mushroom){
	int i;
	for(i = 0; i < 90; i++){
		iowrite16(mushroom->mushroom[i], BG(dev.virtbase) + 2 * i);
	}
	dev.mushroom = *mushroom;
}
static void write_title(vga_ball_title *title){
	int i;
	for(i = 0; i < 40; i++){
		iowrite16(title->title[i], title(dev.virtbase) + 2 * i);
	}
	dev.title = *title;
}

static void write_centipede(vga_ball_centipede *centipede){
	int i;
	for(i = 0; i < 20; i++){
		iowrite16(centipede->centipede[i], C(dev.virtbase) + 2 * i);
	}
	dev.centipede = *centipede;
}

static void write_state(vga_ball_state *state){
	int i;
	for(i = 0; i < 10; i++){
		iowrite16(state->state[i], state(dev.virtbase) + 2 * i);
	}
	dev.state = *state;
}

static void write_bullet(vga_ball_bullet *bullet){
	int i;
	for(i = 0; i < 60; i++){
		iowrite16(bullet->bullet[i], B(dev.virtbase) + 2 * i);
	}
	dev.bullet = *bullet;
}

static void write_player(vga_ball_object *object){
	iowrite16(object->x, P(dev.virtbase));
	iowrite16(object->y, P(dev.virtbase) + 2);
	dev.player = *object;
}

static void write_spider(vga_ball_object *object){
	iowrite16(object->x, S(dev.virtbase));
	iowrite16(object->y, S(dev.virtbase) + 2);
	dev.spider = *object;
}

static void write_start(vga_ball_start *start){
        iowrite16(start->x, start(dev.virtbase));
        dev.start = *start;
}

static void read_mouse(vga_ball_mouse *object){

	object -> x =  ioread16(xlatch(dev.virtbase));
	object -> y =  ioread16(ylatch(dev.virtbase));
        object -> l =  ioread16(leflatch(dev.virtbase));
	object -> r =  ioread16(riglatch(dev.virtbase));
        object -> m =  ioread16(midlatch(dev.virtbase));
//        *object = dev.mouse;

      }

/*
static void write_ball(vga_ball_coordinate *coordinate)
{
	iowrite8(coordinate->x, HCOUNT(dev.virtbase));
	iowrite8(coordinate->y, VCOUNT(dev.virtbase));
	dev.coordinate = *coordinate;
}
*/



/*
 * Handle ioctl() calls from userspace:
 * Read or write the segments on single digits.
 * Note extensive error checking of arguments
 */
static long vga_ball_ioctl(struct file *f, unsigned int cmd, unsigned long arg)
{
	//vga_ball_arg_t vla;
	vga_ball_background mushroom;
	vga_ball_title title;
    vga_ball_centipede centipede;
    vga_ball_state state;
    vga_ball_bullet bullet;
    vga_ball_object player;
    vga_ball_object spider;
    vga_ball_mouse mouse;
    vga_ball_start start;
	switch (cmd) {
	case VGA_BALL_WRITE_BACKGROUND:
		if (copy_from_user(&mushroom, (vga_ball_background *) arg,
				   sizeof(vga_ball_background)))
			return -EACCES;
				write_background(&mushroom);
		break;
	case VGA_BALL_WRITE_TITLE:
		if(copy_from_user(&title, (vga_ball_title *) arg,
				   sizeof(vga_ball_title)))
			return -EACCES;
				write_title(&title);
		break;
	
	case VGA_BALL_WRITE_CENTIPEDE:
		if (copy_from_user(&centipede, (vga_ball_centipede *) arg,
				   sizeof(vga_ball_centipede)))
			return -EACCES;
				write_centipede(&centipede);
		break;
	
	case VGA_BALL_WRITE_STATE:
		if(copy_from_user(&state, (vga_ball_state *) arg,
				    sizeof(vga_ball_state)))
			return -EACCES;
				write_state(&state);
		break;
	
	case VGA_BALL_WRITE_BULLET:
		if (copy_from_user(&bullet, (vga_ball_bullet *) arg,
				   sizeof(vga_ball_bullet)))
			return -EACCES;
				write_bullet(&bullet);
		break;
	
	case VGA_BALL_WRITE_PLAYER:
		if (copy_from_user(&player, (vga_ball_object *) arg,
				   sizeof(vga_ball_object)))
			return -EACCES;
				write_player(&player);
		break;

	case VGA_BALL_WRITE_SPIDER:
		if (copy_from_user(&spider, (vga_ball_object *) arg,
				   sizeof(vga_ball_object)))
			return -EACCES;
				write_spider(&spider);
		break;

	case VGA_BALL_READ_MOUSE:
		read_mouse(&mouse);
		if (copy_to_user((vga_ball_mouse *)arg, &mouse,
			sizeof(vga_ball_mouse)))
			return -EACCES;
			
		break;
        case VGA_BALL_WRITE_START:
                if (copy_from_user(&start, (vga_ball_start*)arg,
                        sizeof(vga_ball_start)))
                        return -EACCES;
				write_start(&start);

                break;


	/*case VGA_BALL_READ_BACKGROUND:
	  	vla.background = dev.background;
		if (copy_to_user((vga_ball_arg_t *) arg, &vla,
				 sizeof(vga_ball_arg_t)))
			return -EACCES;
		break;
		*/
	
	default:
		return -EINVAL;
	}

	return 0;
}

/* The operations our device knows how to do */
static const struct file_operations vga_ball_fops = {
	.owner		= THIS_MODULE,
	.unlocked_ioctl = vga_ball_ioctl,
};

/* Information about our device for the "misc" framework -- like a char dev */
static struct miscdevice vga_ball_misc_device = {
	.minor		= MISC_DYNAMIC_MINOR,
	.name		= DRIVER_NAME,
	.fops		= &vga_ball_fops,
};

/*
 * Initialization code: get resources (registers) and display
 * a welcome message
 */
static int __init vga_ball_probe(struct platform_device *pdev)
{
        //vga_ball_color_t beige = { 0xf9, 0xe4, 0xb7 };
	int ret;

	/* Register ourselves as a misc device: creates /dev/vga_ball */
	ret = misc_register(&vga_ball_misc_device);

	/* Get the address of our registers from the device tree */
	ret = of_address_to_resource(pdev->dev.of_node, 0, &dev.res);
	if (ret) {
		ret = -ENOENT;
		goto out_deregister;
	}

	/* Make sure we can use these registers */
	if (request_mem_region(dev.res.start, resource_size(&dev.res),
			       DRIVER_NAME) == NULL) {
		ret = -EBUSY;
		goto out_deregister;
	}

	/* Arrange access to our registers */
	dev.virtbase = of_iomap(pdev->dev.of_node, 0);
	if (dev.virtbase == NULL) {
		ret = -ENOMEM;
		goto out_release_mem_region;
	}
        
	/* Set an initial color */
        //write_background(&beige);

	return 0;

out_release_mem_region:
	release_mem_region(dev.res.start, resource_size(&dev.res));
out_deregister:
	misc_deregister(&vga_ball_misc_device);
	return ret;
}

/* Clean-up code: release resources */
static int vga_ball_remove(struct platform_device *pdev)
{
	iounmap(dev.virtbase);
	release_mem_region(dev.res.start, resource_size(&dev.res));
	misc_deregister(&vga_ball_misc_device);
	return 0;
}

/* Which "compatible" string(s) to search for in the Device Tree */
#ifdef CONFIG_OF
static const struct of_device_id vga_ball_of_match[] = {
	{ .compatible = "csee4840,vga_ball-1.0" },
	{},
};
MODULE_DEVICE_TABLE(of, vga_ball_of_match);
#endif

/* Information for registering ourselves as a "platform" driver */
static struct platform_driver vga_ball_driver = {
	.driver	= {
		.name	= DRIVER_NAME,
		.owner	= THIS_MODULE,
		.of_match_table = of_match_ptr(vga_ball_of_match),
	},
	.remove	= __exit_p(vga_ball_remove),
};

/* Called when the module is loaded: set things up */
static int __init vga_ball_init(void)
{
	pr_info(DRIVER_NAME ": init\n");
	return platform_driver_probe(&vga_ball_driver, vga_ball_probe);
}

/* Calball when the module is unloaded: release resources */
static void __exit vga_ball_exit(void)
{
	platform_driver_unregister(&vga_ball_driver);
	pr_info(DRIVER_NAME ": exit\n");
}

module_init(vga_ball_init);
module_exit(vga_ball_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Stephen A. Edwards, Columbia University");
MODULE_DESCRIPTION("VGA ball driver");
